#include <ClanLib/display.h>
#include <ClanLib/core.h>
#include <algorithm>

#include "scene.h"
#include "mask_object_extractor.h"

Scene::Scene(const std::string &background_file, const std::string &mask_file, const std::string &walk_file)
{
	CL_TargaProvider background_buffer(background_file);
	CL_TargaProvider mask_buffer(mask_file);

	walk_buffer = new CL_TargaProvider(walk_file);
	walk_buffer->lock();
	if (walk_buffer->get_format().get_depth() != 32)
		throw CL_Error("Walk is not 32bit");

	MaskObjectExtractor extractor(background_buffer, mask_buffer);
	objects = extractor.objects;
	
	background = new CL_Surface(&background_buffer);
	mask = new CL_Surface(&mask_buffer);
	walk = new CL_Surface(walk_buffer);
}

Scene::~Scene()
{
	for(unsigned int i=0; i<objects.size(); ++i)
		delete objects[i];

	delete background;
	delete mask;
	delete walk;

	walk_buffer->unlock();
	delete walk_buffer;
}

void Scene::add_object(Object *object)
{
	objects.push_back(object);
	update_objects();
}

void Scene::update_objects()
{
	// Make sure the objects are z-sorted
	std::sort(objects.begin(), objects.end(), object_compare());
}

void Scene::draw()
{
	// Draw background (but skip if 1 is pressed)
	if(!CL_Keyboard::get_keycode(CL_KEY_1))
		background->draw();
	else
		CL_Display::clear();

	// Draw walk if 3 is pressed
	if(CL_Keyboard::get_keycode(CL_KEY_3))
		walk->draw();

	// Draw all extracted mask objects (but skip if 4 is pressed)
	if(!CL_Keyboard::get_keycode(CL_KEY_4))
		for(unsigned int i=0; i<objects.size(); ++i)
			objects[i]->draw();

	// Draw mask if 2 is pressed
	if(CL_Keyboard::get_keycode(CL_KEY_2))
		mask->draw();
}
