#include <ClanLib/core.h>
#include <ClanLib/gui.h>
#include <ClanLib/guistylesilver.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>
#include <ClanLib/gl.h>

// The main application
class App : public CL_ClanApplication
{
	CL_Component *guimanager;
	
public:
	int main(int argc, char** argv)
	{
		// Create a console window for text-output if not available
		CL_ConsoleWindow console("Console");
		console.redirect_stdio();

		try
		{
			CL_SetupCore::init();
			CL_SetupDisplay::init();
			CL_SetupGL::init();
			CL_SetupGUI::init();

			CL_DisplayWindow display("GUI Simple example", 640, 480);
			
			// Create the GUI using the default style
			CL_ResourceManager gui_resources("../../Resources/GUIStyleSilver/gui.xml");
			CL_StyleManager_Silver style(&gui_resources);
			CL_GUIManager gui(&style);
			guimanager = &gui;

			CL_Slot slot_draw = gui.sig_paint().connect(this, &App::on_draw);

			CL_Window window(CL_Rect(10, 10, 300, 300), "Window", &gui);

			// Override default close handler with our own.
			// Its important to notice we use the connect_virtual() function, and
			// not the normal connect() function. This makes sure we are the only
			// one who will get called when sig_close() is activated. Had we used
			// connect() both us and the default close handler would be called.
			CL_Slot slot_close = window.sig_close().connect_virtual(this, &App::on_close);
			
			// Main loop
			while (!CL_Keyboard::get_keycode(CL_KEY_ESCAPE))
			{
				gui.show();

				CL_System::keep_alive();
				CL_Display::flip();
			}

			CL_SetupGL::deinit();
			CL_SetupGUI::deinit();
			CL_SetupDisplay::deinit();
			CL_SetupCore::deinit();
		}
		catch (CL_Error e)
		{
 			std::cout << e.message.c_str() << std::endl;

			// Display console close message and wait for a key
			console.display_close_message();
		}

		return 0;
	}
	
	void on_close(CL_SlotParent_v0 &super)
	{
		if(CL_MessageBox::info(
			"Really quit?",
			"You have expressed a wish to close the window. Are you really sure ?",
			"Yes", "I don't know", "No",
			guimanager) == 0)
		{
			// If the user answers Yes in the MessageBox, we call the default
			// clanGUI handler for close(), which will close the window.
			super();
		}
	}
	
	void on_draw()
	{
		CL_Display::clear();
	}
} app;
