#include <ClanLib/core.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>
#include <ClanLib/gl.h>

#include "cl_writepng.h"

class ScreenShotApp : public CL_ClanApplication
{
public:
	CL_PixelBuffer *take_screen_shot()
	{
		unsigned short width = CL_Display::get_width();
		unsigned short height = CL_Display::get_height();
		
		CL_PixelBuffer *pbuf = new CL_PixelBuffer(width, height, 320*3, CL_PixelFormat::rgb888);
		
		pbuf->lock();
		
		unsigned char *data = static_cast<unsigned char*>(pbuf->get_data());
		
		CL_GraphicContext *gc = CL_Display::get_current_window()->get_gc();
		
		for (int y = 0; y < height; y++)
		{
			for (int x = 0; x < width; x++)
			{
				CL_Color pixel = gc->get_pixel( x, y );

				*data++ = (unsigned char)pixel.get_red();
				*data++ = (unsigned char)pixel.get_green();
				*data++ = (unsigned char)pixel.get_blue();

			}
		}

		pbuf->unlock();

		return pbuf;
	}
	
	virtual int main(int, char **)
	{
		// Create a console window for text-output if not available
		CL_ConsoleWindow console("Console");
		console.redirect_stdio();
		
		try
		{
			CL_SetupCore::init();
			CL_SetupDisplay::init();
			CL_SetupGL::init();
			
			// Set mode: 320x200 16 bpp
			CL_DisplayWindow window("screenshot example", 320, 200, false );

			std::cout << "Press space to take a screenshot" << std::endl;

			// Loop until the user hits escape:
			while (CL_Keyboard::get_keycode(CL_KEY_ESCAPE) == false)
			{
				// Clear the display
				CL_Display::clear();
				
				// Let's draw something
				CL_Display::fill_rect(
					CL_Rect(1,1,200,200),
					CL_Color::ghostwhite );
/*					CL_Gradient(
						CL_Color::red,
						CL_Color::red,
						CL_Color::blue,
						CL_Color::green));*/

				CL_Display::fill_rect(
					CL_Rect(1,1,50,50),
					CL_Color::cyan );

				CL_Display::fill_rect(
					CL_Rect(50,50,100,100),
					CL_Color::magenta );

				CL_Display::fill_rect(
					CL_Rect(150,150,200,200),
					CL_Color::yellow );
				
				// Make sure our updates are displayed
				CL_Display::flip();
				
				// Update keyboard input and handle system events:
				// Exits the loop if ClanLib requests shutdown - for instance if
				// someone closes the window.
				
				CL_System::keep_alive(10);

				// If someone pressed SPACE
				if(CL_Keyboard::get_keycode(CL_KEY_SPACE))
				{
					// Save a screenshot
					std::cout << "Saving png..." << std::endl;
					CL_PixelBuffer *screenshot = take_screen_shot();

					WritePNG::write_png( screenshot, "screenshot.png" );

					delete screenshot;
				}
			}

			CL_SetupDisplay::deinit();
			CL_SetupCore::deinit();
		}
		catch (CL_Error err)
		{
			std::cout << "Error: " << err.message.c_str() << std::endl;
		}

		// Display console close message and wait for a key
		console.display_close_message();

		return 0;
	}
} app;
