/*  $Id: graphic_context_generic.cpp,v 1.4 2003/07/20 22:31:49 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Display/display_precomp.h"
#include "graphic_context_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_GraphicContext_Generic construction:

CL_GraphicContext_Generic::CL_GraphicContext_Generic()
: count_3d(0), ref_count(0)
{
	cliprects.push_front(CL_Rect(0,0,0,0));
	translates.push_front(CL_Point(0,0));
}

CL_GraphicContext_Generic::~CL_GraphicContext_Generic()
{
}

/////////////////////////////////////////////////////////////////////////////
// CL_GraphicContext_Generic attributes:

/////////////////////////////////////////////////////////////////////////////
// CL_GraphicContext_Generic operations:

void CL_GraphicContext_Generic::add_ref()
{
	ref_count++;
}

void CL_GraphicContext_Generic::release_ref()
{
	ref_count--;
	if (ref_count == 0) delete this;
}

void CL_GraphicContext_Generic::begin_3d()
{
	count_3d++;
	if (count_3d == 1) setup_3d();
}

void CL_GraphicContext_Generic::end_3d()
{
	count_3d--;
	if (count_3d == 0) setup_2d();
}

void CL_GraphicContext_Generic::set_cliprect(const CL_Rect &rect)
{
	cliprects.front() = rect;
}

void CL_GraphicContext_Generic::push_cliprect(const CL_Rect &rect)
{
	cliprects.push_front(cliprects.front().calc_union(rect));
}

void CL_GraphicContext_Generic::push_cliprect()
{
	cliprects.push_front(cliprects.front());
}

void CL_GraphicContext_Generic::pop_cliprect()
{
	cliprects.pop_front();
}

void CL_GraphicContext_Generic::set_translate_offset(int x, int y)
{
	translates.pop_front();
	translates.push_front(CL_Point(x, y));
}

void CL_GraphicContext_Generic::push_translate_offset(int delta_x, int delta_y)
{
	CL_Point point = translates.front();
	point.x += delta_x;
	point.y += delta_y;
	translates.push_front(point);
}

void CL_GraphicContext_Generic::push_translate_offset()
{
	translates.push_front(translates.front());
}

void CL_GraphicContext_Generic::pop_translate_offset()
{
	translates.pop_front();
}

/////////////////////////////////////////////////////////////////////////////
// CL_GraphicContext_Generic implementation:
