/*  $Id: gl_state_checker.h,v 1.15 2003/10/14 15:52:12 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_gl_state_checker
#define header_gl_state_checker

#if _MSC_VER > 1000
#pragma once
#endif

#include <stack>
#include "gl_state.h"
#include <GL/gl.h>
#ifndef WIN32
#include <GL/glx.h>
#endif

#ifndef GL_MIRRORED_REPEAT
#define GL_MIRRORED_REPEAT 0x8370
#endif
#ifndef GL_CLAMP_TO_EDGE
#define GL_CLAMP_TO_EDGE 0x812F
#endif
#ifndef GL_CLAMP_TO_BORDER
#define GL_CLAMP_TO_BORDER 0x812D
#endif

class CL_Rect;
class CL_Rectf;
class CL_Pointf;
class CL_Color;

class CL_GLStateChecker
{
//! Construction:
public:
	CL_GLStateChecker();

//! Operations:
public:
	//: Adds a clear screen.
	static void add_clear(
		const CL_Color &color);

	//: Adds drawal of a pixel to the state tree.
	static void add_pixel(
		const CL_GLState &state,
		const CL_Pointf &position);

	//: Adds drawal of a line to the state tree.
	static void add_line(
		const CL_GLState &state,
		const CL_Rectf &position);

	//: Adds drawal of a gradient to the state tree.
	static void add_gradient(
		const CL_GLState &state,
		const CL_Rectf &position,
		const CL_Gradient &gradient);

	//: Adds drawal of a texture to the state tree.
	static void add_quad(
		const CL_GLState &state,
		float px1,
		float py1,
		float px2,
		float py2,
		int tx1,
		int ty1,
		int tx2,
		int ty2);

	//: Renders state tree objects.
	static void flush();

#ifdef WIN32
	// Selects specified context into opengl state machine.
	static void set_context(HDC drawable, HGLRC context);
#else
	// Selects specified context into opengl state machine.
	static void set_context(Display *disp, GLXDrawable drawable, GLXContext context);
#endif

//! Implementation:
private:
	static float calc_rotate_x(const float &px, const float &py)
	{
		return 
			internal_state.rotation_hotspot.x +
			(px-internal_state.rotation_hotspot.x) * rotate_x.x +
			(py-internal_state.rotation_hotspot.y) * rotate_y.x;
	}

	static float calc_rotate_y(const float &px, const float &py)
	{
		return 
			internal_state.rotation_hotspot.y +
			(px-internal_state.rotation_hotspot.x) * rotate_x.y +
			(py-internal_state.rotation_hotspot.y) * rotate_y.y;
	}

	static inline void add_quad_vertices(const CL_GLState &state, float px1, float py1, float px2, float py2 );

	static void set_state(const CL_GLState &state, GLenum primitive);

	static CL_GLState internal_state;

	static GLenum primitive;

	static CL_Pointf rotate_x, rotate_y;

#ifdef WIN32
	//: Active opengl drawable.
	static HDC cur_drawable;

	//: Active opengl context.
	static HGLRC cur_context;
#else
	//: Active opengl drawable.
	static GLXDrawable cur_drawable;

	//: Active opengl context.
	static GLXContext cur_context;
#endif
};

#endif
