/*  $Id: button.cpp,v 1.67 2003/09/18 17:53:29 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "API/GUI/button.h"
#include "API/GUI/stylemanager.h"
#include "API/Core/XML/dom_element.h"
#include "button_generic.h"
#include "component_generic.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_Button::CL_Button(
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(parent, style), impl(0)
{
	impl = new CL_Button_Generic(this, std::string(), false);
	get_style_manager()->connect_styles("button", this);
	find_preferred_size();
}

CL_Button::CL_Button(
	const std::string &text,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(parent, style), impl(0)
{
	impl = new CL_Button_Generic(this, text, false);
	get_style_manager()->connect_styles("button", this);
	find_preferred_size();
}

CL_Button::CL_Button(
	const CL_Rect &pos,
	const std::string &text,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(pos, parent, style), impl(0)
{
	impl = new CL_Button_Generic(this, text, false);
	get_style_manager()->connect_styles("button", this);
}

CL_Button::CL_Button(
	const CL_Point &pos,
	const std::string &text,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(CL_Rect(pos.x, pos.y, 0, 0), parent, style), impl(0)
{
	impl = new CL_Button_Generic(this, text, false);
	get_style_manager()->connect_styles("button", this);
	find_preferred_size();
}

CL_Button::~CL_Button()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

const std::string &CL_Button::get_text() const
{
	return impl->text;
}

bool CL_Button::is_toggle_button() const
{
	return impl->toggle_mode;
}

bool CL_Button::is_down() const
{
	return impl->toggled;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_Button::set_text(const std::string &text)
{
	impl->text = text;
}

void CL_Button::set_toggle_mode(bool toggle)
{
	impl->toggle_mode = toggle;
}

void CL_Button::set_down(bool enable)
{
	impl->toggled = enable;
}

void CL_Button::toggle()
{
	impl->toggled = !impl->toggled;
	impl->sig_toggled(impl->toggled);
}

/////////////////////////////////////////////////////////////////////////////
// Signals:

CL_Signal_v1<bool> &CL_Button::sig_toggled()
{
	return impl->sig_toggled;
}

CL_Signal_v0 &CL_Button::sig_clicked()
{
	return impl->sig_clicked;
}

CL_Signal_v0 &CL_Button::sig_pressed()
{
	return impl->sig_pressed;
}

CL_Signal_v0 &CL_Button::sig_released()
{
	return impl->sig_released;
}

