/*  $Id: popupmenu_boring.cpp,v 1.3 2003/09/08 15:06:24 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "API/Display/display.h"
#include "API/GUI/menunode.h"
#include "API/GUIStyleBoring/popupmenu_boring.h"
#include "API/GUIStyleBoring/stylemanager_boring.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_PopupMenu_Boring::CL_PopupMenu_Boring(
	CL_PopupMenu *popupmenu,
	CL_StyleManager_Boring *style)
:
	CL_ComponentStyle(popupmenu),
	popupmenu(popupmenu),
	style(style)
{
	slot_paint = popupmenu->sig_paint().connect(
		this, &CL_PopupMenu_Boring::on_paint);
	slot_resize = popupmenu->sig_resize().connect(
		this, &CL_PopupMenu_Boring::on_resize);
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_PopupMenu_Boring::on_resize(int old_width, int old_height)
{
	const std::list<CL_MenuNode *> &items = popupmenu->get_items();

	CL_Point point(0, 0);
	int width = popupmenu->get_client_area()->get_width();
	
	std::list<CL_MenuNode *>::const_iterator it;
	for (it = items.begin(); it != items.end(); ++it)
	{
		CL_Component *component = (*it)->get_component();
	
		// Size and position component
		component->set_position(CL_Rect(point.x, point.y, point.x + width, point.y + component->get_height()));

		// Move down for next component
		point.y += component->get_height();
	}
}

void CL_PopupMenu_Boring::on_paint()
{
	//bool focus = popupmenu->has_child(popupmenu->get_focus());

	int width = popupmenu->get_width();
	int height = popupmenu->get_height();
		
	style->draw_box(0, 0, width, height, GUICOLOR_BRIGHT_SHADE, GUICOLOR_DARK_SHADE);
	style->draw_box(1, 1, width - 1, height - 1, GUICOLOR_BRIGHT_SHADE, GUICOLOR_DARK_SHADE);
	style->fill_rect(2, 2, width - 2, height - 2, GUICOLOR_WINDOW_NORMAL);
}
