/*  $Id: server_database.h,v 1.3 2003/10/14 15:52:13 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef header_server_database
#define header_server_database

#if _MSC_VER > 1000
#pragma once
#endif

#include "API/Core/System/mutex.h"
#include "API/Network/Socket/ip_address.h"

#include <string>
#include <deque>
#include <list>
#include <map>

class CL_ServerDescription
{
public:
	std::string desc;
	int ref_count;
};

class CL_ServerUpdate
{
public:
	unsigned int timestamp;
	CL_IPAddress server;
};

class CL_ServerChanges
{
public:
	unsigned int timestamp;
	std::map<CL_IPAddress, CL_ServerDescription *> servers;
	std::list<CL_IPAddress> removed_servers;
};

class CL_ServerDatabase
{
//! Construction:
public:
	//: Constructs a server database.
	CL_ServerDatabase();

	~CL_ServerDatabase();

//! Attributes:
public:

//! Operations:
public:
	//: Increments reference count.
	int add_ref();

	//: Decrements reference count.
	//- <p>When the reference count reaches zero, release_ref will delete the instance.</p>
	int release_ref();

	//: Adds/updates a server in the database.
	void update_server(const CL_IPAddress &address, const std::string &desc);

	//: Removes a server from the database.
	void remove_server(const CL_IPAddress &address);

	//: Returns a list of server changes that has happened since last timestamp.
	CL_ServerChanges get_changes(unsigned int timestamp = 0);

	//: Releases reference counts caused by a call to get_changes.
	void release_changes(CL_ServerChanges &changes);

//! Implementation:
private:
	int ref_count;

	std::map<CL_IPAddress, CL_ServerDescription *> servers;

	std::deque<CL_ServerUpdate> updates;

	CL_Mutex mutex;
};

#endif
