
#ifdef WIN32
#pragma warning(disable : 4355)
#pragma warning(disable : 4786)
#endif

#include <ClanLib/GUI/component_type.h>

#include "propertybar.h"
#include "mainframe.h"
#include "component_node.h"

/////////////////////////////////////////////////////////////////////////////
// PropertyBar construction:

PropertyBar::PropertyBar(MainFrame *mainframe)
: CL_Component(mainframe), mainframe(mainframe), listbox(this), label(this),
  input_text(this), input_bool(this), input_number(this)
{
	Document &doc = get_document();
	
	slots.connect(doc.sig_node_options_changed, this, &PropertyBar::on_node_options_changed);
	slots.connect(mainframe->sig_selection_changed, this, &PropertyBar::on_node_selection_changed);
	slots.connect(sig_resize(), this, &PropertyBar::on_resize);
	slots.connect(listbox.sig_selection_changed(), this, &PropertyBar::on_listbox_selection_changed);
	slots.connect(input_text.sig_return_pressed(), this, &PropertyBar::on_input_change);
	slots.connect(input_bool.sig_return_pressed(), this, &PropertyBar::on_input_change);
	slots.connect(input_number.sig_return_pressed(), this, &PropertyBar::on_input_change);

	on_resize(get_width(), get_height());

	select_none();
}

PropertyBar::~PropertyBar()
{
}

/////////////////////////////////////////////////////////////////////////////
// PropertyBar attributes:

Document &PropertyBar::get_document()
{
	return mainframe->document;
}

/////////////////////////////////////////////////////////////////////////////
// PropertyBar operations:

/////////////////////////////////////////////////////////////////////////////
// PropertyBar callbacks:

void PropertyBar::on_resize(int old_width, int old_height)
{
	int width = get_width();
	int height = get_height();

	listbox.set_position(CL_Rect(4, 4, width-4, height-40));
	label.set_position(CL_Rect(4, height-28, 90, height-14));
	input_text.set_position(CL_Rect(100, height-30, width-4, height-14));
	input_number.set_position(CL_Rect(100, height-30, width-4, height-14));
	input_bool.set_position(CL_Rect(100, height-30, width-4, height-14));
}

void PropertyBar::on_input_change()
{
}

void PropertyBar::on_listbox_selection_changed()
{
	std::string selected_item = listbox.get_current_text();
	
	CL_ComponentType *comp_type = selected_node->find_component_type();
	const std::map<std::string, CL_ComponentType::SOptionType> &type_options = comp_type->get_options();
	std::map<std::string, CL_ComponentType::SOptionType>::const_iterator it;

	input_text.show(false);
	input_number.show(false);
	input_bool.show(false);

	for (it = type_options.begin(); it != type_options.end(); ++it)
	{
		if (selected_item == (*it).first)
		{
			label.set_text("Edit " + (*it).first + ":");

			switch((*it).second.type)
			{
			case CL_ComponentType::EDatatype::STRING:
				input_text.show();
				input_text.set_text("STRING");
				break;
			case CL_ComponentType::EDatatype::BOOL:
				input_bool.show();
				input_bool.set_text("BOOL");
				break;
			case CL_ComponentType::EDatatype::NUMBER:
				input_number.show();
				input_number.set_text("NUMBER");
				break;
			}
		}
	}
}

void PropertyBar::on_node_options_changed(ComponentNode *node)
{
}

void PropertyBar::on_node_selection_changed(std::list<ComponentNode *> &selection)
{
	listbox.clear();
	select_none();

	if (selection.empty())
		return;

	listbox.enable();
	
	selected_node = selection.front();

	// Fetch used component options
	const CL_ComponentOptions &options_used = selected_node->get_options();
	CL_ComponentOptions::const_iterator it_used;

	// Fetch available component options
	CL_ComponentType *comp_type = selected_node->find_component_type();
	const std::map<std::string, CL_ComponentType::SOptionType> &type_options = comp_type->get_options();
	std::map<std::string, CL_ComponentType::SOptionType>::const_iterator it;
	
	for (it = type_options.begin(); it != type_options.end(); ++it)
	{
/*		bool used = false;
		for (it_used = options_used.begin(); it_used != options_used.end(); it_used++)
		{
			if(it->first == it_used->first)
			{
				used = true;
			}
		}
		if(!used)
			listbox.insert_item(it->first + " (default value)");
		else
*/			listbox.insert_item(it->first);
	}
}

/////////////////////////////////////////////////////////////////////////////
// PropertyBar implementation:

void PropertyBar::select_none()
{
	label.set_text("");
	input_text.show(false);
	input_number.show(false);
	input_bool.show(false);
	listbox.enable(false);
	selected_node = NULL;
}

